
#ifndef KPUBLICTRANSPORTONBOARD_EXPORT_H
#define KPUBLICTRANSPORTONBOARD_EXPORT_H

#ifdef KPUBLICTRANSPORTONBOARD_STATIC_DEFINE
#  define KPUBLICTRANSPORTONBOARD_EXPORT
#  define KPUBLICTRANSPORTONBOARD_NO_EXPORT
#else
#  ifndef KPUBLICTRANSPORTONBOARD_EXPORT
#    ifdef KPublicTransportOnboard_EXPORTS
        /* We are building this library */
#      define KPUBLICTRANSPORTONBOARD_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define KPUBLICTRANSPORTONBOARD_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef KPUBLICTRANSPORTONBOARD_NO_EXPORT
#    define KPUBLICTRANSPORTONBOARD_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef KPUBLICTRANSPORTONBOARD_DEPRECATED
#  define KPUBLICTRANSPORTONBOARD_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef KPUBLICTRANSPORTONBOARD_DEPRECATED_EXPORT
#  define KPUBLICTRANSPORTONBOARD_DEPRECATED_EXPORT KPUBLICTRANSPORTONBOARD_EXPORT KPUBLICTRANSPORTONBOARD_DEPRECATED
#endif

#ifndef KPUBLICTRANSPORTONBOARD_DEPRECATED_NO_EXPORT
#  define KPUBLICTRANSPORTONBOARD_DEPRECATED_NO_EXPORT KPUBLICTRANSPORTONBOARD_NO_EXPORT KPUBLICTRANSPORTONBOARD_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef KPUBLICTRANSPORTONBOARD_NO_DEPRECATED
#    define KPUBLICTRANSPORTONBOARD_NO_DEPRECATED
#  endif
#endif

#endif /* KPUBLICTRANSPORTONBOARD_EXPORT_H */
